

//******************************************************************//
//																	//
//				(C) 2017 R-MIK Sawomir Skrzyski					//
//				slawomir.skrzynski@ep.com.pl						//
//																	//
//******************************************************************//


//deklaracja:
//#define	ONEWIRE_OVD_SLOW
//wydua okno czasowe odczytu i stan nieaktywny po zakoczeniu odczytu


//----------------------------------------------------------------------//
//			STD			OVERDRIVE
// tRSTL	480..640	48..80		czas resetu
// tPDH		15..60		2..6		slot testu bitu obecnoci
//
// tSLOT	61...		8...		czas okna bitu
// tW0L		60..120		6..16		zapis bitu 0
// tW1L		1..15		1..2		zapis bitu 1
//
// tRL		5..15		1..2		strob odczytu
// tMSR		tRL + 15	tRL + 2		okno odczytu wartoci bitu


//----------------------------------------------------------------------//
#define	ONEWIRE_OVERDRIVE_SLOW		ONEWIRE_OVD_SLOW


//======================================================================//
byte onewireInit()
{
	#if !defined( ONEWIRE_IRQ ) || ONEWIRE_IRQ>2 || !defined( ONEWIRERTS_IRQ ) || ONEWIRERTS_IRQ>2
	  #error "IRQ trzeba blokowac na czas obslugi 1-Wire"
	#endif


	return( onewireTest() );
}

//----------------------------------------------------------------------//
byte onewireTest()
{
	onewireReset();

	ONEWIRE_PORT |= (1<<ONEWIRE_DQ);		// Lekkie podciganie
	ONEWIRE_DDR &= ~(1<<ONEWIRE_DQ);		// Port na odczyt
#warning "Dodac oczekiwanie (max 4ms) az magistrala przyjmnie 1"
//todo: jestli tak bedzie to to:
//do 3,8ms zgoszenie IRQ przez slave
//ponad - blad magistrali
	nop();
	if ( (ONEWIRE_PIN & (1<<ONEWIRE_DQ)) )
		{
		onewireStrongPullup( TRUE );		// Wcz silne podciganie czyli ustaw "1"
		return( TRUE );
		}


	return( FALSE );						// Wychodzimy bez wczania silnego podcigania bo magistrala zwarta do masy
}


//----------------------------------------------------------------------//
//				Czyta bit
// tSLOT	61...		8...		czas okna bitu
// tRL		5..15		1..2		strob odczytu
// tMSR		tRL + 15	tRL + 2		okno odczytu wartoci bitu
//----------------------------------------------------------------------//
uint8_t onewireReadBit(void)
{
uint8_t dsBit=0;


	onewireStrongPullup( FALSE );		// Wycz silne podciganie

	#if ONEWIRE_IRQ == 0
	  uint8_t cSREG;
	  cSREG = SREG; cli();
	#endif

	//low for 6uS (niektre rda 1us)
	ONEWIRE_PORT &= ~ (1<<ONEWIRE_DQ);	//low
	ONEWIRE_DDR |= (1<<ONEWIRE_DQ); 	//output
	if ( !onewireOverdrive ) _delay_us(6); else  _delay_us(1);

	//release line and wait for 14uS
	ONEWIRE_DDR &= ~(1<<ONEWIRE_DQ); 	//input
	ONEWIRE_PORT |= (1<<ONEWIRE_DQ);	//Lekkie podciganie (gdy brak rezystora zewntrznego)
	if ( !onewireOverdrive ) _delay_us(9);
	else
	#ifndef ONEWIRE_OVD_SLOW
	  _delay_us(1);
	#else
	  _delay_us(5);
	#endif

	//read the value
#warning "nalezaloby umozliwic rozdzielenie pinu wejsciowego od wyjsciowego ja w emulatorze"
	if(ONEWIRE_PIN & (1<<ONEWIRE_DQ)) dsBit=1;

	#if ONEWIRE_IRQ == 0
	  SREG = cSREG;
	#endif

	//wait 45uS and return read value
	if ( !onewireOverdrive ) _delay_us(55);
	else
	#ifndef ONEWIRE_OVD_SLOW
	  _delay_us(7);
	#else
	  _delay_us(20);
	#endif


	return ( dsBit );
}


//----------------------------------------------------------------------//
//				Zapusuje bit
// tSLOT	61...		8...		czas okna bitu
// tW0L		60..120		6..16		zapis bitu 0
// tW1L		1..15		1..2		zapis bitu 1
//----------------------------------------------------------------------//
byte onewireWriteBit(uint8_t bit)
{
	onewireStrongPullup( FALSE );			// Wycz silne podciganie

	#if ONEWIRE_IRQ == 0
	  uint8_t cSREG;
	  cSREG = SREG; cli();
	#endif

	//low for 1uS
	ONEWIRE_PORT &= ~ (1<<ONEWIRE_DQ); 	//low
	ONEWIRE_DDR |= (1<<ONEWIRE_DQ); 	//output
	if ( !onewireOverdrive ) _delay_us(6); else _delay_us(1);

	//if we want to write 1, release the line (if not will keep low)
	if(bit)
		{
		onewireStrongPullup( TRUE );	// Wcz silne podciganie czyli ustaw "1"
//		ONEWIRE_DDR &= ~(1<<ONEWIRE_DQ); //input
		}

	#if ONEWIRE_IRQ == 0
	  	if ( !onewireOverdrive ) SREG = cSREG;	// W trybie overdrive nie przywracamy jeszcze IRQ
	#endif

	//wait 60uS and release the line
	if ( !onewireOverdrive )
		{								// Standard
		if( bit )_delay_us(64-6); else _delay_us(60-6+10);
		}
	else								// Overdrive
		{
		if( bit ) _delay_us(7.5-1); else _delay_us(7.5-1+2.5);
		}

		onewireStrongPullup( TRUE );	// Wcz silne podciganie czyli ustaw "1"
//	ONEWIRE_DDR &= ~(1<<ONEWIRE_DQ); 	//input
	#if ONEWIRE_IRQ == 0
	  SREG = cSREG;						// W trybie standard przywracami IRQ w overdriwe jeszcze raz niepotrzebnie
	#endif
}


//----------------------------------------------------------------------//
//				Generuje reset
// tRSTL		480..640	48..80		czas resetu
// tPDH			30			3
// tpresence	15..60		2..6		slot testu bitu obecnoci
//----------------------------------------------------------------------//
uint8_t onewireReset()
{
uint8_t dsRst;


	onewireRstSt=0;


	#if OONEWIRERTS_IRQ == 1 || ONEWIRERTS_IRQ == 2
	  uint8_t cSREG;
	  cSREG = SREG; cli();
	#endif

	if ( onewireOverdrive ) _delay_us(2.5);

	#if ONEWIRERTS_IRQ == 0
	  uint8_t cSREG;
	  cSREG = SREG;
	  	if ( onewireOverdrive ) cli();	// W trybie overdrive ju zawieszamy IRQ
	#endif

	//low for 480us
	ONEWIRE_PORT &= ~ (1<<ONEWIRE_DQ); 	//low
	ONEWIRE_DDR |= (1<<ONEWIRE_DQ); 	//output
	if ( !onewireOverdrive ) _delay_us( 700 ); else _delay_us( 70 );	// typowo 480us ale niektre 660us

	#if ONEWIRERTS_IRQ == 0
	  	if ( !onewireOverdrive ) cli();	// W trybie standard dopiero teraz zawieszamy IRQ
	#endif

	//release line and wait for 60uS
	ONEWIRE_PORT |= (1<<ONEWIRE_DQ); 	//pulup gdy brak rezystora podcigajcego nie zawiesi si na oczekiwaniu DS18x20 na koniec konwersji
	ONEWIRE_DDR &= ~(1<<ONEWIRE_DQ); 	//input

#warning "dodac timeout"
	//----- Czekamu na koniec impulsu resetu ((zgoszenie IRQ przez slave -----//
word timeout=0;
	while( !(ONEWIRE_PIN & (1<<ONEWIRE_DQ)) )
		{
 		_delay_us(10);
 		if ( ++timeout > 4000/10 ) break;
		}
	if ( !onewireOverdrive ) _delay_us( 70 ); else _delay_us( 9 );

	//get value and wait 420us
	dsRst = (ONEWIRE_PIN & (1<<ONEWIRE_DQ));

	#if ONEWIRERTS_IRQ == 0
	  SREG = cSREG;						// Przywrcenie przerwa
	#endif
	if ( timeout < 3800/10 ) onewireRstSt=2;	// Status resetu 0-OK, 1-presence, 2-alarm, 3-uszkodzona magistrala
	if ( timeout < 960/10 ) onewireRstSt=1;		// Status resetu 0-OK, 1-presence, 2-alarm, 3-uszkodzona magistrala

	if ( !onewireOverdrive ) _delay_us( 480-60 ); else _delay_us( 40);

	onewireStrongPullup( TRUE );		// Wcz silne podciganie
	_delay_ms(1);
	onewireStrongPullup( FALSE );		// Wycz silne podciganie


	return( !dsRst );	// 1-ok, <>0 error
}


//----------------------------------------------------------------------//
